"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createLspConnection = void 0;
const vscode_languageserver_protocol_1 = require("vscode-languageserver-protocol");
/**
 * Wrap JSON RPC `MessageConnection` to provide typed methods for LSP.
 * @param conn
 */
exports.createLspConnection = (conn) => {
    let capabilities;
    // Handle dynamic registrations
    conn.onRequest(vscode_languageserver_protocol_1.RegistrationRequest.type, ({ registrations }) => {
        for (const { method, registerOptions } of registrations) {
            const provider = METHOD_TO_PROVIDER[method];
            if (provider)
                capabilities[provider] = registerOptions !== null && registerOptions !== void 0 ? registerOptions : true;
        }
    });
    conn.onRequest(vscode_languageserver_protocol_1.UnregistrationRequest.type, ({ unregisterations }) => {
        for (const { method } of unregisterations) {
            const provider = METHOD_TO_PROVIDER[method];
            if (provider)
                capabilities[provider] = undefined;
        }
    });
    const maybeReq = (cond, type) => (params) => cond() ? conn.sendRequest(type, params) : Promise.resolve(null);
    const maybeNotify = (cond, type) => (params) => cond() ? conn.sendNotification(type, params) : undefined;
    const notifier = (type) => (params) => conn.sendNotification(type, params);
    const onNotification = (type) => (handler) => {
        conn.onNotification(type, handler);
    };
    const hasTextDocumentWillSave = () => {
        var _a;
        const c = (_a = capabilities.textDocumentSync) !== null && _a !== void 0 ? _a : vscode_languageserver_protocol_1.TextDocumentSyncKind.None;
        return typeof c !== "number" && !!c.willSave;
    };
    const hasTextDocumentWillSaveWaitUntil = () => {
        var _a;
        const c = (_a = capabilities.textDocumentSync) !== null && _a !== void 0 ? _a : vscode_languageserver_protocol_1.TextDocumentSyncKind.None;
        return typeof c !== "number" && !!c.willSaveWaitUntil;
    };
    const hasTextDocumentDidSave = () => {
        var _a;
        const c = (_a = capabilities.textDocumentSync) !== null && _a !== void 0 ? _a : vscode_languageserver_protocol_1.TextDocumentSyncKind.None;
        return typeof c === "number" ? c !== vscode_languageserver_protocol_1.TextDocumentSyncKind.None : !!c.save;
    };
    return {
        /** Start listening. */
        listen: () => conn.listen(),
        /** Dispose connection. */
        dispose: () => conn.dispose(),
        /**
         * Send initialize request with params and store the server capability.
         */
        initialize: async (params) => {
            const result = await conn.sendRequest(vscode_languageserver_protocol_1.InitializeRequest.type, params);
            capabilities = result.capabilities;
            // console.log(result.serverInfo);
            // console.log(JSON.stringify(capabilities, null, 2));
            return result;
        },
        /** Notify that the client initialized. */
        initialized: () => 
        // Need to send empty object literal to prevent ParameterStructures error.
        conn.sendNotification(vscode_languageserver_protocol_1.InitializedNotification.type, {}),
        /** Send shutdown request. */
        shutdown: () => conn.sendRequest(vscode_languageserver_protocol_1.ShutdownRequest.type),
        /** Notify exit. */
        exit: () => conn.sendNotification(vscode_languageserver_protocol_1.ExitNotification.type),
        /** Register an error handler. */
        onError: (handler) => conn.onError(([err, msg, code]) => handler(err, msg, code)),
        /** Register a close handler. */
        onClose: (handler) => conn.onClose(() => handler()),
        /** Register a handler for logMessage notification. */
        onLogMessage: onNotification(vscode_languageserver_protocol_1.LogMessageNotification.type),
        /**
         * Register a handler for showMessage notification.
         * The handler should display the message in the user interface.
         */
        onShowMessage: onNotification(vscode_languageserver_protocol_1.ShowMessageNotification.type),
        /** Register a handler for diagnostics notification. */
        onDiagnostics: onNotification(vscode_languageserver_protocol_1.PublishDiagnosticsNotification.type),
        /**
         * Notify newly opened text document.
         *
         * "open" means the document's truth is now managed by the client and
         * the server must not try to read the document's truth using the document's
         * uri. It doesn't necessarily mean it's presented in an editor.
         *
         * The notification must not be sent more than once without a corresponding
         * close notification sent before.
         */
        textDocumentOpened: notifier(vscode_languageserver_protocol_1.DidOpenTextDocumentNotification.type),
        /**
         * Notify the text document got closed.
         *
         * The document's truth now exists where the document's uri points to
         * and the server is allowed to read it using the document's uri.
         *
         * Requires that the text document is "open".
         */
        textDocumentClosed: notifier(vscode_languageserver_protocol_1.DidCloseTextDocumentNotification.type),
        /**
         * Notify changes to a text document.
         */
        textDocumentChanged: notifier(vscode_languageserver_protocol_1.DidChangeTextDocumentNotification.type),
        /**
         * Notify that the text document will be saved.
         */
        textDocumentWillSave: maybeNotify(hasTextDocumentWillSave, vscode_languageserver_protocol_1.WillSaveTextDocumentNotification.type),
        /**
         * Get text edits to apply before saving.
         */
        getEditsBeforeSave: maybeReq(hasTextDocumentWillSaveWaitUntil, vscode_languageserver_protocol_1.WillSaveTextDocumentWaitUntilRequest.type),
        /**
         * Notify that the text document got saved in the client.
         */
        textDocumentSaved: maybeNotify(hasTextDocumentDidSave, vscode_languageserver_protocol_1.DidSaveTextDocumentNotification.type),
        /**
         * Notify that the client's configuration changed.
         */
        configurationChanged: notifier(vscode_languageserver_protocol_1.DidChangeConfigurationNotification.type),
        /** If supported, request completion at a given text document position. */
        getCompletion: maybeReq(() => !!capabilities.completionProvider, vscode_languageserver_protocol_1.CompletionRequest.type),
        /** If supported, resolve additional information for a given completion item. */
        getCompletionItemDetails: maybeReq(() => { var _a; return !!((_a = capabilities.completionProvider) === null || _a === void 0 ? void 0 : _a.resolveProvider); }, vscode_languageserver_protocol_1.CompletionResolveRequest.type),
        /** If supported, request hover information at a given text document position. */
        getHoverInfo: maybeReq(() => !!capabilities.hoverProvider, vscode_languageserver_protocol_1.HoverRequest.type),
        /** If supported, request signature information at a given text document position. */
        getSignatureHelp: maybeReq(() => !!capabilities.signatureHelpProvider, vscode_languageserver_protocol_1.SignatureHelpRequest.type),
        /** If supported, resolve the type definition locations of a symbol at a given text document position. */
        getDeclaration: maybeReq(() => !!capabilities.declarationProvider, vscode_languageserver_protocol_1.DeclarationRequest.type),
        /** If supported, resolve the definition location of a symbol at a given text document position. */
        getDefinition: maybeReq(() => !!capabilities.definitionProvider, vscode_languageserver_protocol_1.DefinitionRequest.type),
        /** If supported, resolve the type definition locations of a symbol at a given text document position. */
        getTypeDefinition: maybeReq(() => !!capabilities.typeDefinitionProvider, vscode_languageserver_protocol_1.TypeDefinitionRequest.type),
        /** If supported, resolve the implementation locations of a symbol at a given text document position. */
        getImplementation: maybeReq(() => !!capabilities.implementationProvider, vscode_languageserver_protocol_1.ImplementationRequest.type),
        /** If supported, resolve project-wide references for the symbol denoted by the given text document position. */
        getReferences: maybeReq(() => !!capabilities.referencesProvider, vscode_languageserver_protocol_1.ReferencesRequest.type),
        /** If supported, resolve a DocumentHighlight for a given text document position. */
        getDocumentHighlight: maybeReq(() => !!capabilities.documentHighlightProvider, vscode_languageserver_protocol_1.DocumentHighlightRequest.type),
        /** If supported, list all symbols found in a given text document. */
        getDocumentSymbol: maybeReq(() => !!capabilities.documentSymbolProvider, vscode_languageserver_protocol_1.DocumentSymbolRequest.type),
        /** If supported, get commands for the given text document and range. */
        getCodeAction: maybeReq(() => !!capabilities.codeActionProvider, vscode_languageserver_protocol_1.CodeActionRequest.type),
        /** If supported, get code lens for the given text document. */
        getCodeLens: maybeReq(() => !!capabilities.codeLensProvider, vscode_languageserver_protocol_1.CodeLensRequest.type),
        /** If supported, resolve a command for a given code lens. */
        resolveCodeLens: maybeReq(() => { var _a; return !!((_a = capabilities.codeLensProvider) === null || _a === void 0 ? void 0 : _a.resolveProvider); }, vscode_languageserver_protocol_1.CodeLensResolveRequest.type),
        /** If supported, get document links. */
        getDocumentLink: maybeReq(() => !!capabilities.documentLinkProvider, vscode_languageserver_protocol_1.DocumentLinkRequest.type),
        /** If supported, resolve additional information for a given document link. */
        resolveDocumentLink: maybeReq(() => { var _a; return !!((_a = capabilities.documentLinkProvider) === null || _a === void 0 ? void 0 : _a.resolveProvider); }, vscode_languageserver_protocol_1.DocumentLinkResolveRequest.type),
        /** If supported, list all color symbols found in a given text document. */
        getColorSymbols: maybeReq(() => !!capabilities.colorProvider, vscode_languageserver_protocol_1.DocumentColorRequest.type),
        /** If supported, get folding ranges in a document. */
        getFoldingRanges: maybeReq(() => !!capabilities.foldingRangeProvider, vscode_languageserver_protocol_1.FoldingRangeRequest.type),
        /** If supported, get selection ranges in a document. */
        getSelectionRanges: maybeReq(() => !!capabilities.selectionRangeProvider, vscode_languageserver_protocol_1.SelectionRangeRequest.type),
        /** If supported, list project-wide symbols matching the query string in params. */
        getWorkspaceSymbols: maybeReq(() => !!capabilities.workspaceSymbolProvider, vscode_languageserver_protocol_1.WorkspaceSymbolRequest.type),
        get syncsIncrementally() {
            var _a, _b;
            let syncCapability = (_a = capabilities.textDocumentSync) !== null && _a !== void 0 ? _a : vscode_languageserver_protocol_1.TextDocumentSyncKind.None;
            if (typeof syncCapability !== "number") {
                syncCapability = (_b = syncCapability.change) !== null && _b !== void 0 ? _b : vscode_languageserver_protocol_1.TextDocumentSyncKind.None;
            }
            return syncCapability === vscode_languageserver_protocol_1.TextDocumentSyncKind.Incremental;
        },
        get completionTriggers() {
            var _a;
            return ((_a = capabilities.completionProvider) === null || _a === void 0 ? void 0 : _a.triggerCharacters) || [];
        },
        get signatureHelpTriggers() {
            var _a;
            return ((_a = capabilities.signatureHelpProvider) === null || _a === void 0 ? void 0 : _a.triggerCharacters) || [];
        },
        get signatureHelpRetriggers() {
            var _a;
            return ((_a = capabilities.signatureHelpProvider) === null || _a === void 0 ? void 0 : _a.retriggerCharacters) || [];
        },
    };
};
const METHOD_TO_PROVIDER = {
    "textDocument/codeAction": "codeActionProvider",
    "textDocument/codeLens": "codeLensProvider",
    "textDocument/color": "colorProvider",
    "textDocument/completion": "completionProvider",
    "textDocument/declaration": "declarationProvider",
    "textDocument/definition": "definitionProvider",
    "textDocument/documentFormatting": "documentFormattingProvider",
    "textDocument/documentHighlight": "documentHighlightProvider",
    "textDocument/documentLink": "documentLinkProvider",
    "textDocument/documentOnTypeFormatting": "documentOnTypeFormattingProvider",
    "textDocument/documentRangeFormatting": "documentRangeFormattingProvider",
    "textDocument/documentSymbol": "documentSymbolProvider",
    "textDocument/executeCommand": "executeCommandProvider",
    "textDocument/foldingRange": "foldingRangeProvider",
    "textDocument/hover": "hoverProvider",
    "textDocument/implementation": "implementationProvider",
    "textDocument/references": "referencesProvider",
    "textDocument/rename": "renameProvider",
    "textDocument/signatureHelp": "signatureHelpProvider",
    "textDocument/typeDefinition": "typeDefinitionProvider",
    "textDocument/workspaceSymbol": "workspaceSymbolProvider",
};
//# sourceMappingURL=connection.js.map