import type { Message, MessageConnection, NotificationHandler } from "vscode-jsonrpc";
import type { InitializeParams, ProtocolRequestType } from "vscode-languageserver-protocol";
export declare type LspConnection = ReturnType<typeof createLspConnection>;
/**
 * Wrap JSON RPC `MessageConnection` to provide typed methods for LSP.
 * @param conn
 */
export declare const createLspConnection: (conn: MessageConnection) => {
    /** Start listening. */
    listen: () => void;
    /** Dispose connection. */
    dispose: () => void;
    /**
     * Send initialize request with params and store the server capability.
     */
    initialize: (params: InitializeParams) => Promise<import("vscode-languageserver-protocol").InitializeResult<any>>;
    /** Notify that the client initialized. */
    initialized: () => void;
    /** Send shutdown request. */
    shutdown: () => Promise<void>;
    /** Notify exit. */
    exit: () => void;
    /** Register an error handler. */
    onError: (handler: (err: Error, msg?: Message | undefined, code?: number | undefined) => void) => import("vscode-jsonrpc").Disposable;
    /** Register a close handler. */
    onClose: (handler: () => void) => import("vscode-jsonrpc").Disposable;
    /** Register a handler for logMessage notification. */
    onLogMessage: (handler: NotificationHandler<import("vscode-languageserver-protocol").LogMessageParams>) => void;
    /**
     * Register a handler for showMessage notification.
     * The handler should display the message in the user interface.
     */
    onShowMessage: (handler: NotificationHandler<import("vscode-languageserver-protocol").ShowMessageParams>) => void;
    /** Register a handler for diagnostics notification. */
    onDiagnostics: (handler: NotificationHandler<import("vscode-languageserver-protocol").PublishDiagnosticsParams>) => void;
    /**
     * Notify newly opened text document.
     *
     * "open" means the document's truth is now managed by the client and
     * the server must not try to read the document's truth using the document's
     * uri. It doesn't necessarily mean it's presented in an editor.
     *
     * The notification must not be sent more than once without a corresponding
     * close notification sent before.
     */
    textDocumentOpened: (params: import("vscode-languageserver-protocol").DidOpenTextDocumentParams) => void;
    /**
     * Notify the text document got closed.
     *
     * The document's truth now exists where the document's uri points to
     * and the server is allowed to read it using the document's uri.
     *
     * Requires that the text document is "open".
     */
    textDocumentClosed: (params: import("vscode-languageserver-protocol").DidCloseTextDocumentParams) => void;
    /**
     * Notify changes to a text document.
     */
    textDocumentChanged: (params: import("vscode-languageserver-protocol").DidChangeTextDocumentParams) => void;
    /**
     * Notify that the text document will be saved.
     */
    textDocumentWillSave: (params: import("vscode-languageserver-protocol").WillSaveTextDocumentParams) => void;
    /**
     * Get text edits to apply before saving.
     */
    getEditsBeforeSave: (params: import("vscode-languageserver-protocol").WillSaveTextDocumentParams) => Promise<import("vscode-languageserver-protocol").TextEdit[] | null>;
    /**
     * Notify that the text document got saved in the client.
     */
    textDocumentSaved: (params: import("vscode-languageserver-protocol").DidSaveTextDocumentParams) => void;
    /**
     * Notify that the client's configuration changed.
     */
    configurationChanged: (params: import("vscode-languageserver-protocol").DidChangeConfigurationParams) => void;
    /** If supported, request completion at a given text document position. */
    getCompletion: (params: import("vscode-languageserver-protocol").CompletionParams) => Promise<import("vscode-languageserver-protocol").CompletionList | import("vscode-languageserver-protocol").CompletionItem[] | null>;
    /** If supported, resolve additional information for a given completion item. */
    getCompletionItemDetails: (params: import("vscode-languageserver-protocol").CompletionItem) => Promise<import("vscode-languageserver-protocol").CompletionItem | null>;
    /** If supported, request hover information at a given text document position. */
    getHoverInfo: (params: import("vscode-languageserver-protocol").HoverParams) => Promise<import("vscode-languageserver-protocol").Hover | null>;
    /** If supported, request signature information at a given text document position. */
    getSignatureHelp: (params: import("vscode-languageserver-protocol").SignatureHelpParams) => Promise<import("vscode-languageserver-protocol").SignatureHelp | null>;
    /** If supported, resolve the type definition locations of a symbol at a given text document position. */
    getDeclaration: (params: import("vscode-languageserver-protocol").DeclarationParams) => Promise<import("vscode-languageserver-protocol").Location | import("vscode-languageserver-protocol").Location[] | import("vscode-languageserver-protocol").LocationLink[] | null>;
    /** If supported, resolve the definition location of a symbol at a given text document position. */
    getDefinition: (params: import("vscode-languageserver-protocol").DefinitionParams) => Promise<import("vscode-languageserver-protocol").Location | import("vscode-languageserver-protocol").Location[] | import("vscode-languageserver-protocol").LocationLink[] | null>;
    /** If supported, resolve the type definition locations of a symbol at a given text document position. */
    getTypeDefinition: (params: import("vscode-languageserver-protocol").TypeDefinitionParams) => Promise<import("vscode-languageserver-protocol").Location | import("vscode-languageserver-protocol").Location[] | import("vscode-languageserver-protocol").LocationLink[] | null>;
    /** If supported, resolve the implementation locations of a symbol at a given text document position. */
    getImplementation: (params: import("vscode-languageserver-protocol").ImplementationParams) => Promise<import("vscode-languageserver-protocol").Location | import("vscode-languageserver-protocol").Location[] | import("vscode-languageserver-protocol").LocationLink[] | null>;
    /** If supported, resolve project-wide references for the symbol denoted by the given text document position. */
    getReferences: (params: import("vscode-languageserver-protocol").ReferenceParams) => Promise<import("vscode-languageserver-protocol").Location[] | null>;
    /** If supported, resolve a DocumentHighlight for a given text document position. */
    getDocumentHighlight: (params: import("vscode-languageserver-protocol").DocumentHighlightParams) => Promise<import("vscode-languageserver-protocol").DocumentHighlight[] | null>;
    /** If supported, list all symbols found in a given text document. */
    getDocumentSymbol: (params: import("vscode-languageserver-protocol").DocumentSymbolParams) => Promise<import("vscode-languageserver-protocol").DocumentSymbol[] | import("vscode-languageserver-protocol").SymbolInformation[] | null>;
    /** If supported, get commands for the given text document and range. */
    getCodeAction: (params: import("vscode-languageserver-protocol").CodeActionParams) => Promise<(import("vscode-languageserver-protocol").Command | import("vscode-languageserver-protocol").CodeAction)[] | null>;
    /** If supported, get code lens for the given text document. */
    getCodeLens: (params: import("vscode-languageserver-protocol").CodeLensParams) => Promise<import("vscode-languageserver-protocol").CodeLens[] | null>;
    /** If supported, resolve a command for a given code lens. */
    resolveCodeLens: (params: import("vscode-languageserver-protocol").CodeLens) => Promise<import("vscode-languageserver-protocol").CodeLens | null>;
    /** If supported, get document links. */
    getDocumentLink: (params: import("vscode-languageserver-protocol").DocumentLinkParams) => Promise<import("vscode-languageserver-protocol").DocumentLink[] | null>;
    /** If supported, resolve additional information for a given document link. */
    resolveDocumentLink: (params: import("vscode-languageserver-protocol").DocumentLink) => Promise<import("vscode-languageserver-protocol").DocumentLink | null>;
    /** If supported, list all color symbols found in a given text document. */
    getColorSymbols: (params: import("vscode-languageserver-protocol").DocumentColorParams) => Promise<import("vscode-languageserver-protocol").ColorInformation[] | null>;
    /** If supported, get folding ranges in a document. */
    getFoldingRanges: (params: import("vscode-languageserver-protocol").FoldingRangeParams) => Promise<import("vscode-languageserver-protocol").FoldingRange[] | null>;
    /** If supported, get selection ranges in a document. */
    getSelectionRanges: (params: import("vscode-languageserver-protocol").SelectionRangeParams) => Promise<import("vscode-languageserver-protocol").SelectionRange[] | null>;
    /** If supported, list project-wide symbols matching the query string in params. */
    getWorkspaceSymbols: (params: import("vscode-languageserver-protocol").WorkspaceSymbolParams) => Promise<import("vscode-languageserver-protocol").SymbolInformation[] | null>;
    readonly syncsIncrementally: boolean;
    readonly completionTriggers: string[];
    readonly signatureHelpTriggers: string[];
    readonly signatureHelpRetriggers: string[];
};
declare type Result<T> = T extends ProtocolRequestType<any, infer R, any, any, any> ? R : never;
export {};
